//-----------------------------------------------------------------
// EEmsg.cpp: Implementierungsdatei
//-----------------------------------------------------------------
// EEmsg : extended EIB message handler
//-----------------------------------------------------------------

#include "stdafx.h"
#include "EIBnet.h"
#include "EEmsg.h"

#include "EIB.h"
#include "EIBnetCtl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEEmsg

CEEmsg::CEEmsg()
{
   m_pCtrl = NULL;
   m_bConnected = FALSE;

   m_hSemConf = CreateSemaphore(NULL, 0,1, NULL);
}

CEEmsg::~CEEmsg()
{
   CloseHandle(m_hSemConf);
}


//Die folgenden Zeilen nicht bearbeiten. Sie werden vom Klassen-Assistenten bentigt.
#if 0
BEGIN_MESSAGE_MAP(CEEmsg, CSocket)
	//{{AFX_MSG_MAP(CEEmsg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0


/////////////////////////////////////////////////////////////////////////////
// Member-Funktion CEEmsg 

BOOL CEEmsg::Connect(LPCSTR strIpAddr)
{
   BOOL     bOk = FALSE;
   DWORD    dwErr;
   CString  str;

   if (m_bConnected) return (TRUE);

   //-- connect to server

   if (CSocket::Create())
   {
      if (CSocket::Connect(strIpAddr, 0xE11B))
      {
         bOk = TRUE;
      }
      else
      {
         dwErr = GetLastError();
         str.Format("Socket.Connect failed %d", dwErr);
         AfxMessageBox(str, MB_OK | MB_ICONEXCLAMATION );
      }
   }
   else
   {
      dwErr = GetLastError();
      str.Format("Socket.Create failed %d", dwErr);
      AfxMessageBox(str, MB_OK | MB_ICONEXCLAMATION );
   }

   m_byFCB = 0;

   m_bConnected = bOk;
   
   return (bOk);
}


//-----------------------------------------------------------------

BOOL CEEmsg::IsConnected()
{
   return (m_bConnected);
}


//-----------------------------------------------------------------

BOOL CEEmsg::Disconnect()
{
   if (m_bConnected==FALSE) return (TRUE);

   m_pCtrl = NULL;
   CSocket::Close();
   m_bConnected = FALSE;
   return (TRUE);
}


//-----------------------------------------------------------------

void CEEmsg::OnReceive(int nErrorCode)
{
	CSocket::OnReceive(nErrorCode);

   BYTE  pBuf[1500];
   DWORD dwRead;

   dwRead = CSocket::Receive(&pBuf, sizeof(pBuf));

   //--- create logfile
   FILE* logFile;
   logFile = fopen("d:\\eemsg.log", "at");
   if (logFile > 0)
   {
      CString str, st;

      str = CTime::GetCurrentTime().Format("%H:%M:%S ");

      st.Format("rx %2d : ", dwRead);
      str += st;
      for (DWORD i=0; i<dwRead; i++)
      {
         st.Format("%02x ", pBuf[i]);
         str += st;
      }
      str += "\n";
      fwrite(str, sizeof(char), str.GetLength(), logFile);
      fclose(logFile);
   }
   
   
   DWORD dwIdx = 0;     // buffer index

   WORD  wCmd;          // (WORD) command code
   WORD  wLen;          // (WORD) msg data length

   WORD  wAddr;         // GroupAddress
   BYTE  byDataLen;     // EIB Data Length
   DWORD dwDataVal;     // Data value

   do
   {
      wCmd = *((WORD*) pBuf);
      wLen = *((WORD*) pBuf+1);

      switch (wCmd)
      {
         case EIBNET_VALUE_READ:
         case EIBNET_VALUE_RSP:
         case EIBNET_VALUE_WRITE:
  
            //-- get GroupAddr
            wAddr = (pBuf[dwIdx+7] << 8) + pBuf[dwIdx+8];

            //-- get DataLength
            byDataLen = pBuf[dwIdx+9] & 0x0F;
            dwDataVal  = 0xFFFFFFFF;

            //-- get DataValue according to DataLength
            switch (byDataLen)
            {
               case 1: 
                  dwDataVal =  pBuf[dwIdx+11] & 0x3F; 
                  break; 

               case 2: 
                  dwDataVal =  pBuf[dwIdx+12];
                  break; 

               case 3: 
                  dwDataVal = (pBuf[dwIdx+12]<<8) + 
                               pBuf[dwIdx+13]; 
                  break; 

               case 4: 
                  dwDataVal = (pBuf[dwIdx+12]<<16) + 
                              (pBuf[dwIdx+13]<< 8) + 
                               pBuf[dwIdx+14]; 
                  break; 

               case 5: 
                  dwDataVal = (pBuf[dwIdx+12]<<24) + 
                              (pBuf[dwIdx+13]<<16) + 
                              (pBuf[dwIdx+14]<< 8) + 
                               pBuf[dwIdx+15]; 
                  break; 

               default: break;
            }

            //-- notify connected clients
            if (m_pCtrl)
            {
               //-- try this: direct call instead of message (v5)
               m_pCtrl->OnFireEventForThread(wAddr, dwDataVal);
               
               //-- old code
               //PostMessage(m_pCtrl->m_hWnd, WM_RECEIVE_GROUPDATA,
               //            (WPARAM) wAddr, (LPARAM) byDataVal);
            }   
            break;
      
         case EIBNET_CONF:
            //-- confirm
            wAddr = (pBuf[dwIdx+7] << 8) + pBuf[dwIdx+8];
            if (wAddr == m_wWaitForAddr)
               ReleaseSemaphore(m_hSemConf, 1, NULL);
            break;

         default:
            break;      
      }

      //-- 4 bytes header
      dwIdx += (wLen + 4);

   }
   while (dwIdx < dwRead);
}


//-----------------------------------------------------------------

void CEEmsg::OnClose(int nErrorCode)
{
	CSocket::OnClose(nErrorCode);
   m_bConnected = FALSE;
   PostMessage(m_pCtrl->m_hWnd, WM_SOCKET_CLOSE, 0,0);
}


//-----------------------------------------------------------------
// ::SendEIB
//   requires EIBserver v02+
//-----------------------------------------------------------------

BOOL CEEmsg::SendEIB(BYTE *pMsg, DWORD dwToWrite)
{
   BYTE     pBuf[32];                  // tmp. transmit buffer
   DWORD    dwIdx = 0;                 // buffer index

   pBuf[dwIdx++] = LSB(0x8010);
   pBuf[dwIdx++] = MSB(0x8010);

   pBuf[dwIdx++] = LSB((WORD) dwToWrite);
   pBuf[dwIdx++] = MSB((WORD) dwToWrite);

   //-- copy msg into buffer and calc checksums
   for (DWORD i=0; i < dwToWrite; i++)
   {
      pBuf[dwIdx++] = *(pMsg++);
   }

   TRACE("TxMsg->   ");
   for (DWORD t=0; t<dwIdx; t++) TRACE("%02x ", pBuf[t]);
   TRACE("\n");

   //--- create logfile -----------------------
   FILE* logFile;
   logFile = fopen("d:\\eemsg.log", "at");
   if (logFile > 0)
   {
      CString str, st;

      str = CTime::GetCurrentTime().Format("%H:%M:%S ");

      st.Format("tx %2d : ", dwToWrite);
      str += st;
      for (DWORD i=0; i<dwToWrite+4; i++)
      {
         st.Format("%02x ", pBuf[i]);
         str += st;
      }
      str += "\n";
      fwrite(str, sizeof(char), str.GetLength(), logFile);
      fclose(logFile);
   }
   //------------------------------------------


   //-- send this message
   return (CSocket::Send(pBuf, dwToWrite+4));

}

//-----------------------------------------------------------------

BOOL CEEmsg::WaitForConfirm(WORD wAddr)
{
   m_wWaitForAddr = wAddr;
   TRACE("Waiting for %x -", wAddr);

   DWORD dwRet;
   MSG   message;

   //-- wait 100 x 10ms = 1000 ms for confirmation message
   for (int i=0; i<100; i++)
   {   
      dwRet = WaitForSingleObject(m_hSemConf, 10);
      if (dwRet == WAIT_OBJECT_0)
      {
         TRACE("- confirmed\n");
         return TRUE;
      }

      if (::PeekMessage(&message, NULL, 0,0, PM_REMOVE))
      {
         ::TranslateMessage(&message);
         ::DispatchMessage(&message);
      }
   }

   TRACE("- error\n");
   return FALSE;
}

//-----------------------------------------------------------------
