//---------------------------------------------------------------------------
// EIBnetCtl.cpp: Implementierung der CEIBnetCtrl-ActiveX-Steuerelementklasse.
//---------------------------------------------------------------------------
// v01    created
// v02    changed internal socker communication
// v03    added OnClose, GetVersion
// v04    added WaitForConfirm
// v05    bugfix: CEEmsg::OnReceive calls OnFireEventForThread directly and
//        does not use PostMessage any more
// v06.1  added support for 2-Byte EIB messages
//        added GetBuild
//        added SendByte and SendWord
// v06.2  added GetEIS5value()
// v06.3  fixed bug in GetEIS5value()
// v06.4  added SendRaw()-Method
// v07.0  changed receiver from 'short' to 'long'
// v07.1  added support for receiving EIS3 / EIS4 : time and date
// v07.2  added transmitting EIS3 / EIS4
//---------------------------------------------------------------------------

#include "stdafx.h"
#include "EIBnet.h"
#include "EIBnetCtl.h"
#include "EIBnetPpg.h"

#include "EIB.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

#define  myVERSION    7
#define  myBUILD      2

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CEIBnetCtrl, COleControl)

/////////////////////////////////////////////////////////////////////////////
// Nachrichtenzuordnungstabelle

BEGIN_MESSAGE_MAP(CEIBnetCtrl, COleControl)
	//{{AFX_MSG_MAP(CEIBnetCtrl)
	//}}AFX_MSG_MAP
	ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
   ON_MESSAGE(WM_RECEIVE_GROUPDATA, OnFireEventForThread) //custom handler
   ON_MESSAGE(WM_SOCKET_CLOSE, OnFireSocketClose)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch-Tabelle

BEGIN_DISPATCH_MAP(CEIBnetCtrl, COleControl)
	//{{AFX_DISPATCH_MAP(CEIBnetCtrl)
	DISP_FUNCTION(CEIBnetCtrl, "Connect", Connect, VT_BOOL, VTS_BSTR)
	DISP_FUNCTION(CEIBnetCtrl, "Disconnect", Disconnect, VT_EMPTY, VTS_NONE)
	DISP_FUNCTION(CEIBnetCtrl, "SendBit", SendBit, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "ReadObjValue", ReadObjValue, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "IsConnected", IsConnected, VT_BOOL, VTS_NONE)
	DISP_FUNCTION(CEIBnetCtrl, "GetVersion", GetVersion, VT_I2, VTS_NONE)
	DISP_FUNCTION(CEIBnetCtrl, "GetBuild", GetBuild, VT_I2, VTS_NONE)
	DISP_FUNCTION(CEIBnetCtrl, "SendByte", SendByte, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "SendWord", SendWord, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "GetEIS5value", GetEIS5value, VT_R4, VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "SendRaw", SendRaw, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2 VTS_I2 VTS_BSTR)
	DISP_FUNCTION(CEIBnetCtrl, "GetEIStimedate", GetEIStimedate, VT_I2, VTS_I4 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "SendTime", SendTime, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2)
	DISP_FUNCTION(CEIBnetCtrl, "SendDate", SendDate, VT_EMPTY, VTS_I2 VTS_I2 VTS_I2)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Ereignistabelle

BEGIN_EVENT_MAP(CEIBnetCtrl, COleControl)
	//{{AFX_EVENT_MAP(CEIBnetCtrl)
	EVENT_CUSTOM("OnReceive", FireOnReceive, VTS_I2  VTS_I4)
	EVENT_CUSTOM("OnSocketClose", FireOnSocketClose, VTS_NONE)
	//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Eigenschaftenseiten

// ZU ERLEDIGEN: Fgen Sie mehr Eigenschaftenseiten ein, als erforderlich sind.  Denken Sie daran, den Zhler zu erhhen!
BEGIN_PROPPAGEIDS(CEIBnetCtrl, 1)
	PROPPAGEID(CEIBnetPropPage::guid)
END_PROPPAGEIDS(CEIBnetCtrl)


/////////////////////////////////////////////////////////////////////////////
// Klassenerzeugung und GUID initialisieren

IMPLEMENT_OLECREATE_EX(CEIBnetCtrl, "EIBNET.EIBnetCtrl.1",
	0xb8b89503, 0x28f6, 0x11d8, 0xb8, 0x66, 0, 0x60, 0x1d, 0x21, 0xa7, 0xba)


/////////////////////////////////////////////////////////////////////////////
// Typbibliothek-ID und Version

IMPLEMENT_OLETYPELIB(CEIBnetCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Schnittstellen-IDs

const IID BASED_CODE IID_DEIBnet =
		{ 0xb8b89501, 0x28f6, 0x11d8, { 0xb8, 0x66, 0, 0x60, 0x1d, 0x21, 0xa7, 0xba } };
const IID BASED_CODE IID_DEIBnetEvents =
		{ 0xb8b89502, 0x28f6, 0x11d8, { 0xb8, 0x66, 0, 0x60, 0x1d, 0x21, 0xa7, 0xba } };


/////////////////////////////////////////////////////////////////////////////
// Steuerelement-Typinformation

static const DWORD BASED_CODE _dwEIBnetOleMisc =
	OLEMISC_ACTIVATEWHENVISIBLE |
	OLEMISC_SETCLIENTSITEFIRST |
	OLEMISC_INSIDEOUT |
	OLEMISC_CANTLINKINSIDE |
	OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CEIBnetCtrl, IDS_EIBNET, _dwEIBnetOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::CEIBnetCtrlFactory::UpdateRegistry -
// Fgt Eintrge der Systemregistrierung fr CEIBnetCtrl hinzu oder entfernt diese

BOOL CEIBnetCtrl::CEIBnetCtrlFactory::UpdateRegistry(BOOL bRegister)
{
	// ZU ERLEDIGEN: Prfen Sie, ob Ihr Steuerelement den Thread-Regeln nach dem "Apartment"-Modell entspricht.
	// Weitere Informationen finden Sie unter MFC TechNote 64.
	// Falls Ihr Steuerelement nicht den Regeln nach dem Apartment-Modell entspricht, so
	// mssen Sie den nachfolgenden Code ndern, indem Sie den 6. Parameter von
	// afxRegApartmentThreading auf 0 ndern.

	if (bRegister)
		return AfxOleRegisterControlClass(
			AfxGetInstanceHandle(),
			m_clsid,
			m_lpszProgID,
			IDS_EIBNET,
			IDB_EIBNET,
			afxRegApartmentThreading,
			_dwEIBnetOleMisc,
			_tlid,
			_wVerMajor,
			_wVerMinor);
	else
		return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::CEIBnetCtrl - Konstruktor

CEIBnetCtrl::CEIBnetCtrl()
{
	InitializeIIDs(&IID_DEIBnet, &IID_DEIBnetEvents);

	// ZU ERLEDIGEN: Daten der Steuerelementinstanz hier initialisieren.
}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::~CEIBnetCtrl - Destruktor

CEIBnetCtrl::~CEIBnetCtrl()
{
	// ZU ERLEDIGEN: Daten der Steuerelementinstanz hier bereinigen.
}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::OnDraw - Zeichenfunktion

void CEIBnetCtrl::OnDraw(
			CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
	// ZU ERLEDIGEN: Folgenden Code durch eigene Zeichenfunktion ersetzen.

   CBitmap  bm;
   bm.LoadBitmap(IDB_ICON);

   CDC      dcMem;
   dcMem.CreateCompatibleDC(pdc);
   dcMem.SelectObject(&bm);

   pdc->BitBlt(0, 0, 32, 32, &dcMem, 0, 0, SRCCOPY);
}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::DoPropExchange - Untersttzung dauerhafter Eigenschaften

void CEIBnetCtrl::DoPropExchange(CPropExchange* pPX)
{
	ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
	COleControl::DoPropExchange(pPX);

	// ZU ERLEDIGEN: PX_ Funktionen fr jede dauerhafte benutzerdefinierte Eigenschaft aufrufen.

}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl::OnResetState - Setzt das Steuerelement in den Standardzustand zurck

void CEIBnetCtrl::OnResetState()
{
	COleControl::OnResetState();  // Setzt die Standards zurck, die in DoPropExchange gefunden wurden

	// ZU ERLEDIGEN: Andere Steuerelementzustnde hier zurcksetzen.
}


/////////////////////////////////////////////////////////////////////////////
// CEIBnetCtrl Behandlungsroutinen fr Nachrichten 

BOOL CEIBnetCtrl::Connect(LPCTSTR IpAddr) 
{
   m_EEmsg.Connect(IpAddr);
   m_EEmsg.m_pCtrl = this;
	return TRUE;
}

void CEIBnetCtrl::Disconnect() 
{
   m_EEmsg.Disconnect();
}


//-----------------------------------------------------------------
// EVENT: when GroupData is received - pass to subscriber

LRESULT CEIBnetCtrl::OnFireEventForThread(WPARAM wParam, LPARAM lParam)
{
   FireOnReceive(wParam, (long) lParam);

   return 0L;
}


//-----------------------------------------------------------------
// EVENT: when socket is closed

LRESULT CEIBnetCtrl::OnFireSocketClose(WPARAM wParam, LPARAM lParam)
{
   FireOnSocketClose();

   return 0L;
}


//-----------------------------------------------------------------

void CEIBnetCtrl::SendBit(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C, short Value) 
{
   //---------------------------------------------------------------------------
   // Prototype of EIB Message with Msg.Code
   //---------------------------------------------------------------------------
   //               L_DATA.req
   //                  |   Control Field
   //                  |     |    own Adress
   //                  |     |     |     |    P-A/G-Addr
   //                  |     |     |     |     |     |   NCPI
   //                  |     |     |     |     |     |     |   TCPI   Val  res
   //                  |     |     |     |     |     |     |     |     |    |
   //---------------------------------------------------------------------------
   BYTE txMsg[32] = {0x11, 0x0c, 0x11, 0xFF, 0xFF, 0xFF, 0xe1, 0x00, 0x80, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   txMsg[8] += (BYTE) (Value & 0xFF);

   m_EEmsg.SendEIB(txMsg, 9);
   m_EEmsg.WaitForConfirm(GroupAddr);
}


//-----------------------------------------------------------------

void CEIBnetCtrl::SendByte(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C, short Value) 
{
   BYTE txMsg[32] = {0x11, 0x0c, 0x11, 0xFF, 0xFF, 0xFF, 0xe2, 0x00, 0x80, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   txMsg[9] = (BYTE) (Value & 0xFF);

   m_EEmsg.SendEIB(txMsg, 10);
   m_EEmsg.WaitForConfirm(GroupAddr);
}


//-----------------------------------------------------------------

void CEIBnetCtrl::SendWord(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C, short Value) 
{
   BYTE txMsg[32] = {0x11, 0x0c, 0x11, 0xFF, 0xFF, 0xFF, 0xe3, 0x00, 0x80, 0x00, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   txMsg[9]  = (BYTE) (MSB(Value));
   txMsg[10] = (BYTE) (LSB(Value));

   m_EEmsg.SendEIB(txMsg, 11);
   m_EEmsg.WaitForConfirm(GroupAddr);
}


//-----------------------------------------------------------------

void CEIBnetCtrl::SendRaw(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C, short Length, LPCTSTR RawData) 
{
   BYTE txMsg[32];

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   for (int i=0; i<Length; i++)
   {
      txMsg[i] = RawData[i];
   }

   m_EEmsg.SendEIB(txMsg, Length);
   m_EEmsg.WaitForConfirm(GroupAddr);
}


//-----------------------------------------------------------------

void CEIBnetCtrl::ReadObjValue(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C) 
{
   //------------     LDr   CF   ..own PA..  ..PA/GA...  NPCI  TPCI  TPCI
   BYTE txMsg[32] = {0x11, 0x0C, 0x11, 0xFF, 0xFF, 0xFF, 0xe1, 0x00, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   m_EEmsg.SendEIB(txMsg, 9);
   m_EEmsg.WaitForConfirm(GroupAddr);
}


//-----------------------------------------------------------------

BOOL CEIBnetCtrl::IsConnected() 
{
	return (m_EEmsg.IsConnected());
}

//-----------------------------------------------------------------

short CEIBnetCtrl::GetVersion() 
{
	return (myVERSION);
}

short CEIBnetCtrl::GetBuild() 
{
	return (myBUILD);

	return 0;
}

//-----------------------------------------------------------------
// GetEIS5value
//    Einen 2-Byte Wert entsprechend EIS5 in Float umrechnen
//    siehe EIBA Handbook Vol.3 Part.7 Chapt.1
//    "EIB Interworking Standard" Paragraph 2.6.6
//-----------------------------------------------------------------

float CEIBnetCtrl::GetEIS5value(short value) 
{
   int nSign = (value & 0x8000 ? -1 : 1);
   int nExpo = ((value & 0x7800) >> 11);
   int nMant = (value & 0x07FF);

   if (nSign==-1) nMant |= 0xFFFFF800;
   float fValue = (0.01f * nMant) * (1 << nExpo);

	return fValue;
}


//-----------------------------------------------------------------
// Konvertierungsroutinen fr EIS3-EIS4-Datentypen
//
//    Typ               In          Out               Code
//
//    EIS3     Time     3 Byte      4 x 1 Byte        30
//    EIS4     Date     3 Byte      3 x 1 Byte        400
//
//-----------------------------------------------------------------


short CEIBnetCtrl::GetEIStimedate(long value, short select) 
{
   switch (select)
   {
      case 300:
         //-- day of week (1=mon...7=sun)
         return ((value >> 21) & 0x07);
         break;

      case 301:
         //-- hour (0...23)
         return ((value >> 16) & 0x1F);
         break;
         
      case 302:
         //-- minute (0...59)
         return ((value >> 8) & 0x3F);
         break;
         
      case 303:
         //-- second (0...59)
         return (value & 0x3F);
         break;
         
      case 400:
         //-- day (1...31)
         return ((value >> 16) & 0x1F);
         break;
         
      case 401:
         //-- month (1...12)
         return ((value >> 8) & 0x0F);
         break;
         
      case 402:
         //-- year (1...12)
         int nYear = value & 0x7F;
         if ( (nYear >= 90) && (nYear <= 99)) return (1900+nYear);
         if ( (nYear >=  0) && (nYear <= 89)) return (2000+nYear);
         return (-1);
         break;
   
   }

	return (-1);      // Fehler
}


//-----------------------------------------------------------------
// SendTime
//
//    aktuelle Systemzeit in EIS3 umrechnen und versenden
//-----------------------------------------------------------------

void CEIBnetCtrl::SendTime(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C) 
{

   //-- Prototype of message
   BYTE  txMsg[32] = {0x11, 0x0c, 0x11, 0xFF, 0x53, 0x00, 0xe1, 0x00, 0x80, 0x00, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   BYTE  b1=0;    // dddhhhhh    d: (1=mon...7=sun) h: (0..23)
   BYTE  b2=0;    // 00mmmmmm    m: (0..59)
   BYTE  b3=0;    // 00ssssss    s: (0..59)

   BYTE  dow;     // day of week

   time_t ltime;
   struct tm *tmNow;
   time (&ltime);
   tmNow = localtime (&ltime);

   //-- adjust day of week: sunday
   dow = tmNow->tm_wday;
   if (dow == 0) dow = 7;

   //-- compose data bytes
   b1 = (dow << 5) | (tmNow->tm_hour & 0x1F);
   b2 = tmNow->tm_min & 0x3F;
   b3 = tmNow->tm_sec & 0x3F;

   //-- put into EIB message
   txMsg[6] = 0xE4;
   txMsg[9]  = b1;
   txMsg[10] = b2;
   txMsg[11] = b3;

   m_EEmsg.SendEIB(txMsg, 12);
   m_EEmsg.WaitForConfirm(GroupAddr);

   return;
}


//-----------------------------------------------------------------
// SendDate
//
//    aktuelles Systemdatum in EIS3 umrechnen und versenden
//-----------------------------------------------------------------

void CEIBnetCtrl::SendDate(short GroupAddr_A, short GroupAddr_B, short GroupAddr_C) 
{
   //-- Prototype of message
   BYTE  txMsg[32] = {0x11, 0x0c, 0x11, 0xFF, 0x53, 0x00, 0xe1, 0x00, 0x80, 0x00, 0x00};

   EIBGA GroupAddr = GA(GroupAddr_A, GroupAddr_B, GroupAddr_C);

   txMsg[4] = MSB(GroupAddr);
   txMsg[5] = LSB(GroupAddr);

   BYTE  b1=0;    // 000DDDDD    D: (1..31)
   BYTE  b2=0;    // 0000MMMM    M: (1..12)
   BYTE  b3=0;    // 0YYYYYYY    Y: (0=2000..89=2089)

   time_t ltime;
   struct tm *tmNow;
   time (&ltime);
   tmNow = localtime (&ltime);

   //-- compose data bytes
   b1 = tmNow->tm_mday & 0x1F;
   b2 = (tmNow->tm_mon+1) & 0x0F;
   b3 = (tmNow->tm_year-100) & 0x7F;

   //-- put into EIB message
   txMsg[6] = 0xE4;
   txMsg[9]  = b1;
   txMsg[10] = b2;
   txMsg[11] = b3;

   m_EEmsg.SendEIB(txMsg, 12);
   m_EEmsg.WaitForConfirm(GroupAddr);

   return;
}
